/* wrProxyBackendLib.h - proxy backend interface header */

/*
 * Copyright (c) 2005-2006 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */
 
/*
modification history
--------------------
01b,16jun06,dbt  Added routine to validate backend arguments.
		 Added full prototype for function pointers (cleaner).
01a,15apr05,dbt  written
*/

#ifndef __INCwrProxyBackendLibh
#define __INCwrProxyBackendLibh

#ifdef __cplusplus
extern "C" {
#endif

#include "wrProxyLibP.h"

/* typedefs */

typedef	int (*PROXY_BKEND_OPEN_RTN) (PROXY_CONNECTION * pProxy,
					char * args, char * errorMsg,
					int errorMsgLength, BOOL dryRun);
typedef int (*PROXY_BKEND_READ_RTN) (PROXY_CONNECTION * pProxy,
					char * buffer, int bufferLength);
typedef int (*PROXY_BKEND_WRITE_RTN) (PROXY_CONNECTION * pProxy,
					char * buffer, int bufferLength);
typedef void (*PROXY_BKEND_CLOSE_RTN) (PROXY_CONNECTION * pProxy);

typedef struct proxy_backend
    {
    struct proxy_backend * next;		/* pointer on next backend */
    						/* structure */
    char *		name;			/* backend name */
    PROXY_BKEND_OPEN_RTN	openRtn;	/* backend open routine */
    PROXY_BKEND_READ_RTN	readRtn;	/* backend read routine */
    PROXY_BKEND_WRITE_RTN	writeRtn;	/* backend write routine */
    PROXY_BKEND_CLOSE_RTN	closeRtn;	/* backend close routine */
    } PROXY_BACKEND;

/* function declarations */

STATUS	wrProxyBackendRegister	(const char * backendName,
				 PROXY_BKEND_OPEN_RTN openRtn,
				 PROXY_BKEND_READ_RTN readRtn,
				 PROXY_BKEND_WRITE_RTN writeRtn,
				 PROXY_BKEND_CLOSE_RTN closeRtn);
int	wrProxyBackendConnect	(PROXY_CONNECTION * pProxy,
					char * args, char * errorMsg,
					int errorMsgLength, BOOL dryRun);
int	wrProxyBackendRead	(PROXY_CONNECTION * pProxy, char * buffer,
					int bufLength);
int	wrProxyBackendWrite	(PROXY_CONNECTION * pProxy, char * buffer,
					int nBytes);
void	wrProxyBackendClose	(PROXY_CONNECTION * pProxy);

#ifdef __cplusplus
}
#endif

#endif	/* __INCwrProxyBackendLibh */
