/* wrProxyLibP.h - private proxy interface header */

/*
 * Copyright (c) 2005-2006 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */
 
/*
modification history
--------------------
01b,20jun06,dbt  Added <pServer> list in proxy structure
		 Removed pProxyList global variable.
01a,15apr05,dbt  written
*/

#ifndef __INCwrProxyLibPh
#define __INCwrProxyLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* Connection status */

#define	PROXY_STATUS_CONNECTED	1	/* proxy connected to target */

/* defines */

#ifdef WIN32
#define	CLOSE_SOCKET		closesocket
#else /* WIN32 */
#define	CLOSE_SOCKET		close
#endif /* WIN32 */

/* Proxy connection mode */

typedef enum proxy_mode 
    {
    PROXY_MODE_RAW	= 0,	/* raw mode */
    PROXY_MODE_PACKET	= 1	/* packet mode */
    } PROXY_MODE;

/* Proxy connection protocol */

typedef enum proxy_protocol
    {
    PROXY_PROT_RAW		= 0,	/* raw protocol */
    PROXY_PROT_WDBSERIAL	= 1	/* wdbserial protocol */
    } PROXY_PROTOCOL;

/* Proxy maximum packet size */

#define	PROXY_MAX_PACKET_SIZE	4000

/* typedefs */

typedef struct proxy_connection
    {
    void *			next;		/* next proxy connection */
    						/* in list */
    void *			prev;		/* previous proxy connection */
    						/* in list */
    int				clientFd;	/* client file descriptor */
    int				status;		/* connection status */
    int				tgtFd;		/* target file descriptor */
    struct proxy_backend *	pBackend;	/* backend used by this */
    						/* connection */
    void *			backendArgs;	/* connection specific */
    						/* backend args */
    PROXY_MODE			mode;		/* connection mode (raw, */
    						/* packet) */
    PROXY_PROTOCOL		protocol;	/* connection protocol */
    char *			buffer;		/* buffer used for wdbserial */
    						/* protocol */
    int				bufIx;		/* buffer index */
    struct proxy_server *	pServer;	/* server that has been used */
    						/* to establish connection */
    } PROXY_CONNECTION;

/* function declarations */

int	wrProxyClientCommandArgGet (char * args, const char * argName,
					int * pIntResult,
					char * pCharResult, UINT32 length);

#ifdef __cplusplus
}
#endif

#endif	/* __INCwrProxyLibPh */
