/* wrProxyWdbSerialProtLib.c - proxy WDB serial protocol library */

/*
 * Copyright (c) 2005 Wind River Systems, Inc.  
 *
 * The right to copy, distribute, modify or otherwise make use  of this 
 * software may be licensed only pursuant to the terms of an applicable Wind 
 * River license agreement.
 */

/*
modification history
--------------------
01a,20may05,dbt  written.
*/

/*
DESCRIPTION

This library contains the core of the Target Agent Proxy.
*/

/* includes */

#ifndef	HOST
#include <vxWorks.h>
#else	/* !HOST */
#include <host.h>
#endif	/* HOST */

#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifdef	HOST
#include <unistd.h>
#include <sys/time.h>
#ifndef	WIN32
#   include <strings.h>
#   include <sys/ioctl.h>
#   include <sys/socket.h>
#else	/* WIN32 */
#   include "rpc/rpc.h"
#   include "io.h"
#endif	/* WIN32 */
#else	/* HOST */
#include <sockLib.h>
#include <selectLib.h>
#endif	/* HOST */
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include "wrProxyLibP.h"
#include "wrProxyBackendLib.h"
#include "wrProxyWdbSerialProtLib.h"

/* defines */

/* SLIP protocol characters */

#define FRAME_END          (char)0300   /* Frame End */
#define FRAME_ESCAPE       (char)0333   /* Frame Esc */
#define TRANS_FRAME_END    (char)0334   /* transposed frame end */
#define TRANS_FRAME_ESCAPE (char)0335   /* transposed frame esc */

/* dummy UDP/IP header info (in 16-bit words) */

#define UDP_IP_HDR_SIZE	28

#define IP_VERS_LEN	0	/* IP version + IP header length */
#define IP_PROT		9	/* protocol to use */
#define IP_DEST_ADDR	12	/* destination address */
#define IP_SRC_ADDR	16	/* source address */
#define UDP_SRC_PORT	20	/* source port */
#define UDP_DEST_PORT	22	/* destination port */
#define UDP_LEN		24	/* length */
#define UDP_CKSUM	26	/* checksum */

#define IN_BUF(offset)							\
		    ((offset) < UDP_IP_HDR_SIZE ?			\
				    pUdpIpHdr [offset] :		\
				    inBuf [offset - UDP_IP_HDR_SIZE])

#define BUF_PUT(ch)							\
		    if (bytesRead >= UDP_IP_HDR_SIZE) 			\
			{						\
			outBuf[writeIx] = ch;				\
			writeIx ++;					\
			}

#ifdef  IPPROTO_UDP
#undef 	IPPROTO_UDP
#endif
#define IPPROTO_UDP	0x11

#define WDBPORT	(unsigned short)0x4321

/*******************************************************************************
*
* wrProxyWdbSerialProtEncode - encode a WDB request for WDB serial protocol
*
* This routine encodes a WDB request for WDB serial protocol.
*
* RETURNS: number of bytes in the encoded buffer
*/


int wrProxyWdbSerialProtEncode
    (
    char *	inBuf,			/* input buffer */
    int		inBufLength,		/* input buffer length */
    char *	outBuf,			/* pointer on output buffer */
    int		outBufLength		/* output buffer length */
    )
    {
    char	buf [UDP_IP_HDR_SIZE];	/* temporary buffer to store UDP hdr */
    char *	pUdpIpHdr;		/* pointer on UDP header */
    int		outputIx;		/* output buffer index */
    int		inputIx;		/* input buffer index */

    /* set up a dummy UDP/IP header */

    pUdpIpHdr	= buf;

    bzero (pUdpIpHdr, UDP_IP_HDR_SIZE);

    pUdpIpHdr [IP_VERS_LEN]			= 0x45;
    pUdpIpHdr [IP_PROT]				= IPPROTO_UDP;
    * (short *) &pUdpIpHdr [UDP_DEST_PORT]	= htons(WDBPORT);

    outBuf[0] = FRAME_END;
    outputIx = 1;

    for (inputIx = 0; inputIx < inBufLength + UDP_IP_HDR_SIZE; inputIx ++)
	{
	if (outputIx >= outBufLength)
	    return (-1);

	switch (IN_BUF (inputIx))
	    {
	    case FRAME_END:
		outBuf [outputIx++] = FRAME_ESCAPE;
		outBuf [outputIx++] = TRANS_FRAME_END;
		break;

	    case FRAME_ESCAPE:
		outBuf [outputIx++] = FRAME_ESCAPE;
		outBuf [outputIx++] = TRANS_FRAME_ESCAPE;
		break;

	    default:
		outBuf [outputIx++] = IN_BUF(inputIx);
	    }
	}

    outBuf [outputIx++] = FRAME_END;

    return (outputIx);
    }

/*******************************************************************************
*
* wrProxyWdbSerialProtIsFrameEnd - check if character is frame end
*
* This routine checks if specified character is frame end
*
* RETURNS: TRUE or FALSE
*/

BOOL wrProxyWdbSerialProtIsFrameEnd
    (
    char	inChar	/* character to verify */
    )
    {
    if (inChar == FRAME_END)
    	return (TRUE);
    else
    	return (FALSE);
    }

/*******************************************************************************
*
* wrProxyWdbSerialProtDecode - decode a WDB request for WDB serial protocol
*
* This routine decods a WDB request for WDB serial protocol.
*
* RETURNS: number of bytes in the decoded buffer or ERROR if data could not
*	   be decoded.
*/

int wrProxyWdbSerialProtDecode
    (
    char *	inBuf,			/* input buffer */
    int		inBufLength,		/* input buffer length */
    char *	outBuf,			/* output buffer */
    int		outBufLength		/* output buffer length */
    )
    {
    int	readIx;			/* read index */
    int	writeIx = 0;		/* write index */
    int	bytesRead = 0;		/* number of read bytes including UDP header */

    /* Note that we do not need an extra buffer to decode data */

    for (readIx = 0; readIx < inBufLength; readIx ++)
    	{
	switch (inBuf[readIx])
	    {
	    case FRAME_END:
	    	if (bytesRead == 0)
		    {
		    /* This is the first frame character */

		    continue;
		    }

		/* This is the last frame character */

		return (writeIx);
		break;

	    case FRAME_ESCAPE:
	    	readIx ++;
		switch (inBuf[readIx])
		    { 
		    case TRANS_FRAME_END:
			BUF_PUT (FRAME_END);
			break;

		    case TRANS_FRAME_ESCAPE:
			BUF_PUT (FRAME_ESCAPE);
			break;

		    default:
			BUF_PUT (inBuf[readIx]);
		    }

		bytesRead ++;	/* one more character has been read */
		break;

	    default:
		BUF_PUT (inBuf[readIx]);
		bytesRead ++;	/* one more character has been read */
	    }
	}
    /* Return the number of bytes written in the buffer */

    return (writeIx);
    }
